@@ -1,18 +1,17 @@
+# -*- coding: utf-8 -*-
 """
 Generators for random graphs
 
 """
-#    Copyright (C) 2004,2005 by 
+#    Copyright (C) 2004-2008 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    Distributed under the terms of the GNU Lesser General Public License
 #    http://www.gnu.org/copyleft/lesser.html
 __author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult(dschult@colgate.edu)"""
-__date__ = "$Date: 2005-06-17 08:06:22 -0600 (Fri, 17 Jun 2005) $"
-__credits__ = """"""
-__revision__ = "$Revision: 1049 $"
 import random
+import math
 import networkx
 from networkx.generators.classic import empty_graph, path_graph, complete_graph
 
@@ -20,79 +19,219 @@ from networkx.generators.classic import empty_graph, path_graph, complete_graph
 #  Some Famous Random Graphs
 #-------------------------------------------------------------------------
 
-def binomial_graph(n,p,seed=None):
+
+def fast_gnp_random_graph(n, p, seed=None):
     """
-    Return a binomial random graph G_{n,p}.
+    Return a random graph G_{n,p}.
+
+    The G_{n,p} graph choses each of the possible [n(n-1)]/2 edges
+    with probability p.
+
+    Sometimes called Erdős-Rényi graph, or binomial graph.
 
     :Parameters:
       - `n`: the number of nodes
-      - `p`: probability that any given edge exist
+      - `p`: probability for edge creation
       - `seed`: seed for random number generator (default=None)
       
+    This algorithm is O(n+m) where m is the expected number of
+    edges m=p*n*(n-1)/2.
+    
+    It should be faster than gnp_random_graph when p is small, and
+    the expected number of edges is small, (sparse graph).
+
+    See:
+
+    Batagelj and Brandes, "Efficient generation of large random networks",
+    Phys. Rev. E, 71, 036113, 2005.
+
     """
     G=empty_graph(n)
-    G.name="Binomial Graph"
+    G.name="fast_gnp_random_graph(%s,%s)"%(n,p)
 
     if not seed is None:
         random.seed(seed)
 
-    for u in xrange(1,n+1):
-        for v in xrange(u+1,n+1):
-            if random.random() < p: G.add_edge(u,v)
+    v=1  # Nodes in graph are from 0,n-1 (this is the second node index).
+    w=-1
+    lp=math.log(1.0-p)  
+
+    while v<n:
+        lr=math.log(1.0-random.random())
+        w=w+1+int(lr/lp)
+        while w>=v and v<n:
+            w=w-v
+            v=v+1
+        if v<n:
+            G.add_edge(v,w)
     return G
 
-def erdos_renyi_graph(n,m,seed=None):
+
+def gnp_random_graph(n, p, seed=None):
     """
-    Return the Erdos-Renyi random graph  G_{n,m}.
+    Return a random graph G_{n,p}.
+
+    Choses each of the possible [n(n-1)]/2 edges with probability p.
+    This is the same as binomial_graph and erdos_renyi_graph. 
+
+    Sometimes called Erdős-Rényi graph, or binomial graph.
+
+    :Parameters:
+      - `n`: the number of nodes
+      - `p`: probability for edge creation
+      - `seed`: seed for random number generator (default=None)
+      
+    This is an O(n^2) algorithm.  For sparse graphs (small p) see
+    fast_gnp_random_graph. 
+
+    P. Erdős and A. Rényi, On Random Graphs, Publ. Math. 6, 290 (1959).
+    E. N. Gilbert, Random Graphs, Ann. Math. Stat., 30, 1141 (1959).
+
+    """
+    G=empty_graph(n)
+    G.name="gnp_random_graph(%s,%s)"%(n,p)
+
+    if not seed is None:
+        random.seed(seed)
+
+    for u in xrange(n):
+        for v in xrange(u+1,n):
+            if random.random() < p:
+                G.add_edge(u,v)
+    return G
+
+# add some aliases to common names
+binomial_graph=gnp_random_graph
+erdos_renyi_graph=gnp_random_graph
+
+def dense_gnm_random_graph(n, m, seed=None):
+    """
+    Return the random graph G_{n,m}.
+
+    Gives a graph picked randomly out of the set of all graphs
+    with n nodes and m edges.
+    This algorithm should be faster than gnm_random_graph for dense graphs.
 
     :Parameters:
       - `n`: the number of nodes
       - `m`: the number of edges
       - `seed`: seed for random number generator (default=None)
-      
-    """
-    G=empty_graph(n)
-    G.name="Erdos-Renyi Graph"
 
-    if not seed is None:
+
+    Algorithm by Keith M. Briggs Mar 31, 2006.
+    Inspired by Knuth's Algorithm S (Selection sampling technique),
+    in section 3.4.2 of
+
+    The Art of Computer Programming by Donald E. Knuth
+    Volume 2 / Seminumerical algorithms
+    Third Edition, Addison-Wesley, 1997.
+ 
+    """
+    mmax=n*(n-1)/2
+    if m>=mmax:
+        G=complete_graph(n)
+    else:
+        G=empty_graph(n)
+
+        G.name="dense_gnm_random_graph(%s,%s)"%(n,m)
+  
+    if n==1 or m>=mmax:
+        return G
+  
+    if seed is not None:
         random.seed(seed)
 
-    if n==1: return G
+    u=0
+    v=1
+    t=0
+    k=0
+    while True:
+        if random.randrange(mmax-t)<m-k:
+            G.add_edge(u,v)
+            k+=1
+            if k==m: return G
+        t+=1
+        v+=1
+        if v==n: # go to next row of adjacency matrix
+            u+=1
+            v=u+1
 
-    if m>=n*(n-1)/2: return complete_graph(n)
+def gnm_random_graph(n, m, seed=None):
+    """
+    Return the random graph G_{n,m}.
+
+    Gives a graph picked randomly out of the set of all graphs
+    with n nodes and m edges.
+
+    :Parameters:
+        - `n`: the number of nodes
+        - `m`: the number of edges
+        - `seed`: seed for random number generator (default=None)
+    """
+    G=empty_graph(n)
+    G.name="gnm_random_graph(%s,%s)"%(n,m)
+
+    if seed is not None:
+        random.seed(seed)
+
+    if n==1:
+        return G
+
+    if m>=n*(n-1)/2:
+        return complete_graph(n)
 
     nlist=G.nodes()
     edge_count=0
     while edge_count < m:
-        # generate random edge
+        # generate random edge,u,v
         u = random.choice(nlist)
         v = random.choice(nlist)
         if u==v or G.has_edge(u,v):
             continue
+        # is this faster?
+        # (u,v)=random.sample(nlist,2)
+        #  if G.has_edge(u,v):
+        #      continue
         else:
             G.add_edge(u,v)
             edge_count=edge_count+1
     return G
 
-def newman_watts_strogatz_graph(n,k,p,seed=None):
+def newman_watts_strogatz_graph(n, k, p, seed=None):
     """
     Return a Newman-Watts-Strogatz small world graph.
 
-    The graph is a ring with k neighbors with new edges (shortcuts)
-    *added* randomly with probability p for each edge.  No edges
-    are removed.
+    First create a ring over n nodes.  Then each node in the ring is
+    connected with its k nearest neighbors (k-1 neighbors if k is odd).  
+    Then shortcuts are created by adding new edges as follows: 
+    for each edge u-v in the underlying "n-ring with k nearest neighbors" 
+    with probability p add a new edge u-w with randomly-chosen existing 
+    node w.  In contrast with watts_strogatz_graph(), no edges are removed.
 
     :Parameters:
       - `n`: the number of nodes
-      - `k`: each vertex is connected to k neighbors in the circular topology
+      - `k`: each node is connected to k nearest neighbors in ring topology
       - `p`: the probability of adding a new edge for each edge
       - `seed`: seed for random number generator (default=None)
+
+
+    Reference::
+
+    @ARTICLE{newman-1999-263,
+    author = {M.~E.~J. Newman and D.~J. Watts},
+    title = {Renormalization group analysis of the small-world network model},
+    journal = {Physics Letters A},
+    volume = {263},
+    pages = {341},
+    url = {http://www.citebase.org/abstract?id=oai:arXiv.org:cond-mat/9903357},
+    year = {1999}
+    }
       
     """
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)
     G=empty_graph(n)
-    G.name="Newmann-Watts-Strogatz Graph"
+    G.name="newman_watts_strogatz_graph(%s,%s,%s)"%(n,k,p)
     nlist = G.nodes()
     fromv = nlist
     # connect the k/2 neighbors
@@ -100,36 +239,44 @@ def newman_watts_strogatz_graph(n,k,p,seed=None):
         tov = fromv[n:] + fromv[0:n] # the first n are now last
         for i in range(len(fromv)):
             G.add_edge(fromv[i], tov[i])
-    # randomly connect nodes with probability p
-    e = G.edges()
+    # for each edge u-v, with probability p, randomly select existing
+    # node w and add new edge u-w 
+    e = G.edges() 
     for (u, v) in e:
         if random.random() < p:
-            v = random.choice(nlist)
-            # no G loops and we want a new edge
+            w = random.choice(nlist)
+            # no self-loops and reject if edge u-w exists
             # is that the correct NWS model?
-            while v == u or G.has_edge(u, v): 
-                v = random.choice(nlist)
-            G.add_edge(u,v)
+            while w == u or G.has_edge(u, w): 
+                w = random.choice(nlist)
+            G.add_edge(u,w)
     return G            
 
-def watts_strogatz_graph(n,k,p,seed=None):
+def watts_strogatz_graph(n, k, p, seed=None):
     """
     Return a Watts-Strogatz small world graph.
 
-    The graph is a ring with k neighbors with
-    edges rewired randomly with probability p.
+    First create a ring over n nodes.  Then each node in the ring is
+    connected with its k nearest neighbors (k-1 neighbors if k is odd).  
+    Then shortcuts are created by rewiring existing edges as follows: 
+    for each edge u-v in the underlying "n-ring with k nearest neighbors" 
+    with probability p replace u-v with a new edge u-w with 
+    randomly-chosen existing node w. In contrast with
+    newman_watts_strogatz_graph(), the random rewiring does not
+    increase the number of edges.
+    
 
     :Parameters:
       - `n`: the number of nodes
-      - `k`: each vertex is connected to k neighbors in the circular topology
+      - `k`: each node is connected to k neighbors in the ring topology
       - `p`: the probability of rewiring an edge
       - `seed`: seed for random number generator (default=None)
       
     """
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)
     G=empty_graph(n)
-    G.name="Watts-Strogatz Graph"
+    G.name="watts_strogatz_graph(%s,%s,%s)"%(n,k,p)
     nlist = G.nodes()
     fromv = nlist
     # connect the k/2 neighbors
@@ -137,22 +284,23 @@ def watts_strogatz_graph(n,k,p,seed=None):
         tov = fromv[n:] + fromv[0:n] # the first n are now last
         for i in range(len(fromv)):
             G.add_edge(fromv[i], tov[i])
-    # randomly rewire nodes with probability p
+    # for each edge u-v, with probability p, randomly replace with
+    # edge u-w
     e = G.edges()
     for (u, v) in e:
         if random.random() < p:
             newv = random.choice(nlist)
-            # no G loops and we want a new edge
+            # avoid self-loops and reject if edge u-newv exists
             # is that the correct WS model?
             while newv == u or G.has_edge(u, newv): 
                 newv = random.choice(nlist)
-            G.delete_edge(u,v)
+            G.delete_edge(u,v)  # conserve number of edges 
             G.add_edge(u,newv)
     return G            
 
 
 
-def random_regular_graph(d,n,seed=None):
+def random_regular_graph(d, n, seed=None):
     """Return a random regular graph of n nodes each with degree d, G_{n,d}.
     Return False if unsuccessful.
     
@@ -161,7 +309,7 @@ def random_regular_graph(d,n,seed=None):
     Nodes are numbered 0...n-1. 
     To get a uniform sample from the space of random graphs
     you should chose d<n^{1/3}.
-    .
+    
     For algorith see Kim and Vu's paper.
 
     Reference::
@@ -202,47 +350,40 @@ def random_regular_graph(d,n,seed=None):
     if not n*d%2==0:
         raise networkx.NetworkXError, "n * d must be even"
 
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)    
 
-    G=empty_graph(0)
-
-    nlist=range(0,n)
-    G.add_nodes_from(nlist)
+        
+    G=empty_graph(n)
+    G.name="random_regular_graph(%s,%s)"%(d,n)
 
     # keep track of the edges we have seen
     seen_edges={}
-    [seen_edges.setdefault(v,{}) for v in nlist]
+    [seen_edges.setdefault(v,{}) for v in range(n)]
 
-    vv=[ [v]*d for v in nlist ]   # List of degree-repeated vertex numbers
+    vv=[ [v]*d for v in range(n)]   # List of degree-repeated vertex numbers
     stubs=reduce(lambda x,y: x+y ,vv)  # flatten the list of lists to a list
 
     while len(stubs) > 0:
-       source=random.choice(stubs)
-       target=random.choice(stubs)
-       if source!=target and not seen_edges[source].has_key(target):
-           stubs.remove(source)
-           stubs.remove(target)
-           seen_edges[source][target]=1
-           seen_edges[target][source]=1
-           G.add_edge(source,target)
-       else:
-           # further check to see if suitable 
-           s=_suitable(stubs)
-           if not s:                   
-#               print "Warning: not a regular graph"
-#               print "unsuitable stubs left",stubs," with existing edges:"
-#               for s in stubs:
-#                   print s,G.edges(s)
-               return False
-               
-       
+        source=random.choice(stubs)
+        target=random.choice(stubs)
+        if source!=target and not seen_edges[source].has_key(target):
+            stubs.remove(source)
+            stubs.remove(target)
+            seen_edges[source][target]=1
+            seen_edges[target][source]=1
+            G.add_edge(source,target)
+        else:
+            # further check to see if suitable 
+            s=_suitable(stubs)
+            if not s:                   
+                return False
     return G
 
 
 
-def barabasi_albert_graph(n,m,seed=None):
-    """Return random graph using Barabasi-Albert preferential attachment model.
+def barabasi_albert_graph(n , m, seed=None):
+    """Return random graph using Barabási-Albert preferential attachment model.
     
     A graph of n nodes is grown by attaching new nodes
     each with m edges that are preferentially attached
@@ -250,50 +391,52 @@ def barabasi_albert_graph(n,m,seed=None):
     
     :Parameters:
       - `n`: the number of nodes
-      - `m`: average degree and must be an positive integer
+      - `m`: number of edges to attach from a new node to existing nodes
       - `seed`: seed for random number generator (default=None)
 
-    The initialization is a graph with with m nodes.
+    The initialization is a graph with with m nodes and no edges.
 
     Reference::
 
-      @ARTICLE{barabasi-1999-emergence,
-      TITLE = {Emergence of scaling in random networks},
-      AUTHOR = {A. L. Barabasi and R. Albert},
-      JOURNAL = {SCIENCE},
-      VOLUME = {286},
-      NUMBER = {5439},
-      PAGES = {509 -- 512},
-      MONTH = {OCT},
-      YEAR = {1999},
+      @article{barabasi-1999-emergence,
+      title   = {Emergence of scaling in random networks},
+      author  = {A. L. Barabási and R. Albert},
+      journal = {Science},
+      volume  = {286},
+      number  = {5439},
+      pages   = {509 -- 512},
+      year = {1999},
       }
 
 
     """
         
-    if m < 1 or n < m:
+    if m < 1 or  m >=n:
         raise networkx.NetworkXError,\
-              "NetworkXError must have m>1 and m<n, m=%d,n=%d"%(m,n)
+              "Barabási-Albert network must have m>=1 and m<n, m=%d,n=%d"%(m,n)
 
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)    
 
-    G=empty_graph(0)
-    edge_targets=range(0,m)  
-    G.add_nodes_from(edge_targets) # add m initial nodes (m0 in barabasi-speak)
-    # list of existing nodes, with nodes repeated once for each adjacent edge 
-    existing_nodes=[]
-    source=m            # next node is m
-    while source<n:     # Now add the other n-1 nodes
-        G.add_edges_from(zip([source]*m,edge_targets))
-        existing_nodes.extend(edge_targets) 
-        existing_nodes.extend([source]*m)
+    G=empty_graph(m)       # add m initial nodes (m0 in barabasi-speak)
+    G.name="barabasi_albert_graph(%s,%s)"%(n,m)
+    edge_targets=range(m)  # possible targets for new edges
+    repeated_nodes=[]      # list of existing nodes,
+                           # with nodes repeated once for each adjacent edge 
+    source=m               # next node is m
+    while source<n:        # Now add the other n-m nodes
+        G.add_edges_from(zip([source]*m,edge_targets)) # add links to m nodes
+        repeated_nodes.extend(edge_targets) # add one node for each new link
+        repeated_nodes.extend([source]*m) # and new node "source" has m links
         # choose m nodes randomly from existing nodes
-        edge_targets=random.sample(existing_nodes,m) 
+        # N.B. during each step of adding a new node the probabilities
+        # are fixed, is this correct? or should they be updated.
+        # Also, random sampling prevents some parallel edges. 
+        edge_targets=random.sample(repeated_nodes,m) 
         source += 1
     return G
 
-def powerlaw_cluster_graph(n,m,p,seed=None):
+def powerlaw_cluster_graph(n, m, p, seed=None):
     """
     Holme and Kim algorithm for growing graphs with powerlaw
     degree distribution and approximate average clustering. 
@@ -318,51 +461,74 @@ def powerlaw_cluster_graph(n,m,p,seed=None):
 
     The average clustering has a hard time getting above 
     a certain cutoff that depends on m.  This cutoff is often quite low.
+    Note that the transitivity (fraction of triangles to possible
+    triangles) seems to go down with network size. 
 
-    It is essentially the Barabasi-Albert growth model with an
+    It is essentially the Barabási-Albert growth model with an
     extra step that each random edge is followed by a chance of
     making an edge to one of its neighbors too (and thus a triangle).
     
     This algorithm improves on B-A in the sense that it enables a
-    higher average clustering to be attained if desired. The largest
-    average clustering seems to be independent of n and attained with
-    m=1 and p=1 (cc=0.74 or so).
+    higher average clustering to be attained if desired. 
+
+    It seems possible to have a disconnected graph with this algorithm
+    since the initial m nodes may not be all linked to a new node
+    on the first iteration like the BA model.
 
     """
-    if not seed is None:
+
+    if m < 1 or n < m:
+        raise networkx.NetworkXError,\
+              "NetworkXError must have m>1 and m<n, m=%d,n=%d"%(m,n)
+
+    if p > 1 or p < 0:
+        raise networkx.NetworkXError,\
+              "NetworkXError p must be in [0,1], p=%f"%(p)
+
+
+    if seed is not None:
         random.seed(seed)    
 
-    G=empty_graph(0)
+    G=empty_graph(m)       # add m initial nodes (m0 in barabasi-speak)
+    G.name="Powerlaw-Cluster Graph"
+    repeated_nodes=G.nodes()  # list of existing nodes to sample from
+                           # with nodes repeated once for each adjacent edge 
+    source=m               # next node is m
+    while source<n:        # Now add the other n-1 nodes
+        possible_targets=random.sample(repeated_nodes,m)
+        # do one preferential attachment for new node
+        target=possible_targets.pop()
+        G.add_edge(source,target)  
+        repeated_nodes.append(target) # add one node to list for each new link
+        count=1
+        while count<m:  # add m-1 more new links
+            if random.random()<p: # clustering step: add triangle 
+                neighborhood=[nbr for nbr in G.neighbors(target) \
+                               if not G.has_edge(source,nbr) \
+                               and not nbr==source]
+                if neighborhood: # if there is a neighbor without a link
+                    nbr=random.choice(neighborhood)
+                    G.add_edge(source,nbr) # add triangle
+                    repeated_nodes.append(nbr) 
+                    count=count+1
+                    continue # go to top of while loop
+            # else do preferential attachment step if above fails
+            target=possible_targets.pop()
+            G.add_edge(source,target) 
+            repeated_nodes.append(target) 
+            count=count+1
 
-    # initialize
-    lookupstub=[0]
-    # add m initial nodes (m0 in barabasi-speak)
-    m0=m
-    G.add_nodes_from(range(0,m0))
-    count=m0
-    # initialize
-    lookupstub=range(0,m0)
-    # Now add the other n-1 nodes
-    while count<n:
-        for mm in xrange(m):
-            pick=random.choice(lookupstub)
-            lookupstub.extend([pick,count])
-            if random.random()<p:
-                nbrs=G.neighbors(pick)
-                if nbrs:   # any neighbors?
-                    pickpick=random.choice(G.neighbors(pick))
-                    lookupstub.extend([pickpick,count])
-                    G.add_edge(count,pickpick)
-            G.add_edge(count,pick)
-        count += 1
+        repeated_nodes.extend([source]*m)  # add source node to list m times
+        source += 1
     return G
 
-def random_lobster(n,p1,p2,seed=None):
+def random_lobster(n, p1, p2, seed=None):
     """Return a random lobster.
 
      A caterpillar is a tree that reduces to a path graph when pruning
-     all leave nodes. A lobster is a tree that reduces to a caterpillar
-     when pruning all leave nodes.
+     all leaf nodes (p2=0).
+     A lobster is a tree that reduces to a caterpillar when pruning all
+     leaf nodes.
      
     :Parameters:
       - `n`: the expected number of nodes in the backbone
@@ -370,28 +536,25 @@ def random_lobster(n,p1,p2,seed=None):
       - `p2`: probability of adding an edge one level beyond backbone
       - `seed`: seed for random number generator (default=None)
 
-"""
-    if not seed is None:
+    """
+    # a necessary ingredient in any self-respecting graph library
+    if seed is not None:
         random.seed(seed)
-    L=networkx.Graph()
-    len=int(2*random.random()*n + 0.5)
-    L=path_graph(len)
-    L.name="random_lobster(%d, %5.3f, %5.3f)"%(n,p1,p2)
+    llen=int(2*random.random()*n + 0.5)
+    L=path_graph(llen)
+    L.name="random_lobster(%d,%s,%s)"%(n,p1,p2)
     # build caterpillar: add edges to path graph with probability p1
-    current_node=len
-    for n in xrange(1,len+1):
-        if random.random()<p1:
-            current_node+=1
-            L.add_edge(n,current_node)
-    # build lobster
-    len2=current_node
-    for n in xrange(len,len2+1):
-        if random.random()<p2:
+    current_node=llen-1
+    for n in xrange(llen):
+        if random.random()<p1: # add fuzzy caterpillar parts
             current_node+=1
             L.add_edge(n,current_node)
+            if random.random()<p2: # add crunchy lobster bits
+                current_node+=1
+                L.add_edge(current_node-1,current_node)
     return L # voila, un lobster!
 
-def random_shell_graph(constructor,seed=None):
+def random_shell_graph(constructor, seed=None):
     """
     Return a random shell graph for the constructor given.
 
@@ -409,20 +572,20 @@ def random_shell_graph(constructor,seed=None):
 
     """
     G=empty_graph(0)
-    G.name="random_shell_graph"
+    G.name="random_shell_graph(constructor)"
 
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)
 
     glist=[]        
     intra_edges=[]
     nnodes=0
-    # create erdos-renyi graphs for each shell
+    # create gnm graphs for each shell
     for (n,m,d) in constructor:
         inter_edges=int(m*d)
         intra_edges.append(m-inter_edges)
         g=networkx.operators.convert_node_labels_to_integers(
-                     erdos_renyi_graph(n,inter_edges),
+                     gnm_random_graph(n,inter_edges),
                      first_label=nnodes)
         glist.append(g)
         nnodes+=n                     
@@ -469,8 +632,9 @@ def random_powerlaw_tree(n, gamma=3, seed=None, tries=100):
     except:
         raise networkx.NetworkXError,\
               "Exceeded max (%d) attempts for a valid tree sequence."%tries
-
-    return degree_sequence_tree(s)
+    G=degree_sequence_tree(s)
+    G.name="random_powerlaw_tree(%s,%s)"%(n,gamma)
+    return G
 
 
 def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
@@ -488,7 +652,7 @@ def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
       - `seed`: seed for random number generator (default=None)
       
     """
-    if not seed is None:
+    if seed is not None:
         random.seed(seed)
 
     # get trial sequence        
@@ -507,7 +671,8 @@ def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
         index=random.randint(0,n-1)
         zseq[index]=swap.pop()
         
-    raise networkx.NetworkXError, "Exceeded max (%d) attempts for a valid tree sequence."%tries
+    raise networkx.NetworkXError, \
+          "Exceeded max (%d) attempts for a valid tree sequence."%tries
     return False
 
 
@@ -516,15 +681,19 @@ def random_powerlaw_tree_sequence(n, gamma=3, seed=None, tries=100):
 
 def _test_suite():
     import doctest
-    suite = doctest.DocFileSuite('../tests/generators_random_graphs.txt',package='networkx')
+    suite = doctest.DocFileSuite('tests/generators/random_graphs.txt',package='networkx')
     return suite
 
 if __name__ == "__main__":
+    import os
     import sys
     import unittest
     if sys.version_info[:2] < (2, 4):
         print "Python version 2.4 or later required for tests (%d.%d detected)." %  sys.version_info[:2]
         sys.exit(-1)
+    # directory of networkx package (relative to this)
+    nxbase=sys.path[0]+os.sep+os.pardir
+    sys.path.insert(0,nxbase) # prepend to search path
     unittest.TextTestRunner().run(_test_suite())
     
 
